'' =================================================================================================
''
''   File....... jm_servo8_measure.spin
''   Purpose.... Measures up to 8 servo pulses in 10us units
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2010 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 21 DEC 2010
''
'' =================================================================================================


var

  long  cog

  long  basepin
  long  us10
  long  servopos[8]
 

pub init(base) | ok

'' base is starting pin of 8-pin group

  finalize

  basepin := base                                               ' pin for servo group
  us10    := clkfreq / 1_000_000 * 10                           ' ticks per 10 microseconds
  
  ok := cog := cognew(@entry, @basepin) + 1                     ' start the cog

  return ok


pub finalize

'' Stops cog (if running)

  if cog
    cogstop(cog~ - 1)
    
  longfill(@servopos, -1, 8)                                    ' initialize array


pub getpos(ch)

'' Get current position for servo #ch (1 to 8)

  if (ch => 1) and (ch =< 8)                                    ' valid servo?
    return servopos[--ch] * 10                                  ' return microseconds
  else
    return -1


dat

                        org     0

entry                   mov     tmp1, par                       ' start of structure

                        rdlong  svo1, tmp1                      ' read base pin
                        add     tmp1, #4
                        
                        rdlong  tix10, tmp1                     ' read ticks per us
                        add     tmp1, #4
                        
                        mov     hubpntr, tmp1                   ' save hub address of array

                        mov     oldscan, ina                    ' preset old scan

                        mov     timer, tix10                    ' start 10us timer
                        add     timer, cnt                      ' sync

svostart                mov     chan, #0                        ' start with frst
                        mov     chmask, #1                      ' create mask for base pin
                        shl     chmask, svo1

                        mov     newscan, ina                    ' grab new inputs
                        mov     deltascan, oldscan              ' determine changed pins
                        xor     deltascan, newscan

svoloop                 test    newscan, chmask         wc      ' ch status to C flag
                        test    deltascan, chmask       wz      ' Z = no change
                        mov     oldscan, newscan                ' save for next cycle

        if_c_and_nz     jmp     #chreset                        ' 0 -> 1 transition
        if_c_and_z      jmp     #chinc                          ' 1 -> 1                          '
        if_nc_and_nz    jmp     #chupdate                       ' 1 -> 0 transition
        if_nc_and_z     jmp     #chnull                         ' 0 -> 0

chreset                 mov     value, #0                       ' reset channel measurement
                        call    #wrchan                         ' write to cog array
                        jmp     #chnull

chinc                   call    #rdchan                         ' read channel measurement
                        add     value, #1                       ' increment
                        call    #wrchan                         ' write it back (cog)               
                        jmp     #chnull
                       
chupdate                call    #rdchan                         ' read measurement
                        mov     tmp1, hubpntr                   ' point to servopos[0]
                        mov     tmp2, chan                      ' create ch offset 
                        shl     tmp2, #2                        ' << 2 for longs 
                        add     tmp1, tmp2                      ' point to servopos[chan]   
                        wrlong  value, tmp1                     ' write position to hub
                       
chnull                  shl     chmask, #1                      ' update mask for channel
                        add     chan, #1                        ' inc channel count
                        cmp     chan, #8                wc, wz  ' done? (chan == 8)
        if_b            jmp     #svoloop                        ' if no, do next

                        waitcnt timer, tix10                    ' let frame expire
                        jmp     #svostart                       ' start all over


' chan = index of element to read
' value = value that is read

rdchan                  mov     tmp1, #pwidth0
                        add     tmp1, chan                      
                        movs    rdval, tmp1                     ' point to pwidth0[chan]
                        nop
rdval                   mov     value, 0-0                      ' value := pwidth0[chan]  
rdchan_ret              ret


' chan = index of element to write
' value = value to write

wrchan                  mov     tmp1, #pwidth0
                        add     tmp1, chan
                        movd    wrval, tmp1                     ' point to pwidth0[chan]
                        nop
wrval                   mov     0-0, value                      ' pwidth0[chan] := value
wrchan_ret              ret   

' --------------------------------------------------------------------------------------------------

pwidth0                 long    0                               ' pulse width in 10us units 
pwidth1                 long    0                               
pwidth2                 long    0
pwidth3                 long    0
pwidth4                 long    0
pwidth5                 long    0
pwidth6                 long    0
pwidth7                 long    0                             

svo1                    res     1                               ' pin for servo #1
tix10                   res     1                               ' ticks per 10us
hubpntr                 res     1                               ' hub address of timing array

oldscan                 res     1                               ' last scan
newscan                 res     1                               ' current scan
deltascan               res     1                               ' changes

timer                   res     1                               ' for 10us frame timing

chan                    res     1                               ' current channel (0 - 7)
chmask                  res     1                               ' mask for channel

value                   res     1                               ' value for cog arrays

tmp1                    res     1                               ' work vars
tmp2                    res     1

                        fit     492                                    

                        
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  